using System;
using System.Collections.Generic;
using System.Drawing;
using System.Xml;
using System.Web.Script.Serialization;

using MindFusion.Diagramming;
using MindFusion.Diagramming.WebForms;
using MindFusion.Drawing;

using Bitmap=System.Drawing.Bitmap;
using Graphics=System.Drawing.Graphics;
using Image=System.Drawing.Image;


public class IconNode : ShapeNode
{
    static IconNode()
    {
        defaultIcon = new Bitmap(48, 48);

        Graphics graphics = Graphics.FromImage(defaultIcon);
		Font font = new Font("Arial", 48, FontStyle.Bold, GraphicsUnit.Pixel);

        graphics.FillRectangle(Brushes.White, 0, 0, 48, 48);
        graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
        graphics.DrawString("?", font, Brushes.ForestGreen, 0, 0);

        graphics.Dispose();
    }

    public IconNode(IconNode prototype)
        : base(prototype)
    {
        icon = prototype.Icon;
        label = prototype.Label;

        format = new StringFormat();
        format.Alignment = StringAlignment.Center;
        format.LineAlignment = StringAlignment.Center;

		this.Font = new Font("Arial", 3, FontStyle.Bold, GraphicsUnit.Pixel);
        Bounds = prototype.Bounds;
    }

    public IconNode(Diagram diagram)
        : base(diagram)
    {
        icon = defaultIcon;
        label = "label";

        format = new StringFormat();
        format.Alignment = StringAlignment.Center;
        format.LineAlignment = StringAlignment.Center;

		this.Font = new Font("Arial", 3, FontStyle.Bold, GraphicsUnit.Pixel);
        Bounds = new RectangleF(Bounds.Location, CalculateSize());
    }

    public override void DrawShadow(IGraphics graphics, RenderOptions options)
    { }

    public override void Draw(IGraphics graphics, RenderOptions options)
    {
        Rectangle iconSizePixels = new Rectangle(0, 0, icon.Width, icon.Height);
        RectangleF imageSize = MindFusion.Utilities.DeviceToDoc(graphics, iconSizePixels);

        // Draw the icon center at the top
        (icon as Bitmap).MakeTransparent((icon as Bitmap).GetPixel(0, 0));
        graphics.DrawImage(icon, Bounds.X + Bounds.Width / 2 - imageSize.Width / 2, Bounds.Y);

        // Draw the label at the bottom
        RectangleF labelBounds = RectangleF.FromLTRB(
            Bounds.X, Bounds.Y + imageSize.Height, Bounds.Right, Bounds.Bottom);

        graphics.DrawString(label,
            Font, Brushes.Black, labelBounds, format);
    }

    private SizeF CalculateSize()
    {
        Bitmap tempImage = new Bitmap(1, 1);
        Graphics graphics = Graphics.FromImage(tempImage);
        IGraphics measureGraphics = new GdiGraphics(graphics);

        Parent.MeasureUnit.ApplyTo(measureGraphics);
        Rectangle iconSizePixels = new Rectangle(0, 0, icon.Width, icon.Height);
        RectangleF imageSize = MindFusion.Utilities.DeviceToDoc(
            measureGraphics, iconSizePixels);

        measureGraphics.Dispose();
        tempImage.Dispose();

		SizeF textSize = Parent.MeasureString(label, this.Font, int.MaxValue, format);

        return new SizeF(
            Math.Max(imageSize.Width, textSize.Width),
            imageSize.Height + textSize.Height);
    }


    protected override void UpdateCreate(PointF current)
    {
        base.UpdateCreate(current);
        Bounds = new RectangleF(current, CalculateSize());
    }

    protected override void SaveToXml(XmlElement xmlElement, XmlPersistContext context)
    {
        base.SaveToXml(xmlElement, context);
        context.WriteString(label, "Label", xmlElement);
        context.WriteImage(icon, "Image", xmlElement);
    }

    protected override void LoadFromXml(XmlElement xmlElement, XmlPersistContext context)
    {
        base.LoadFromXml(xmlElement, context);
        label = context.ReadString("Label", xmlElement);
        icon = context.ReadImage("Image", xmlElement);
    }

    protected override void SaveTo(System.IO.BinaryWriter writer, PersistContext context)
    {
        base.SaveTo(writer, context);
        writer.Write(label);
        context.SaveImage(icon);
    }

    protected override void LoadFrom(System.IO.BinaryReader reader, PersistContext context)
    {
        base.LoadFrom(reader, context);
        label = reader.ReadString();
        icon = context.LoadImage();
    }


    public Image Icon
    {
        get { return icon; }
        set
        {
            icon = value;
            Bounds = new RectangleF(Bounds.Location, CalculateSize());
        }
    }

    public string Label
    {
        get { return label; }
        set
        {
            label = value;
            Bounds = new RectangleF(Bounds.Location, CalculateSize());
        }
    }


    private Image icon;
    private string label;
    private StringFormat format;
    static private Image defaultIcon;
}

public class IconNodeConverter : ShapeNodeConverter
{
    public IconNodeConverter(DiagramView view)
        : base(view, true)
    {
    }

    public override IEnumerable<Type> SupportedTypes
    {
        get { return new List<Type>(new Type[] { typeof(IconNode) }); }
    }

    public override object Deserialize(
		IDictionary<string, object> dictionary, Type type, JavaScriptSerializer serializer)
    {
		IconNode node = (IconNode)base.Deserialize(dictionary, type, serializer);
        node.Label = dictionary["label"].ToString();
        return node;
    }

    public override IDictionary<string, object> Serialize(
		object obj, JavaScriptSerializer serializer)
    {
        IconNode node = obj as IconNode;
        if (node != null)
        {
            IDictionary<string, object> json = base.Serialize(obj, serializer);
            if (json != null)
            {
                json.Add("label", node.Label);
            }
            return json;
        }

        return null;
    }
}